@@ -1,4 +1,4 @@
-/*  Misfit Model 3D
+/*  Maverick Model 3D
  * 
  *  Copyright (c) 2004-2007 Kevin Worcester
  * 
@@ -38,6 +38,7 @@
 #include "pixmap/crosshairrow.xpm"
 
 #include <QtCore/QTimer>
+#include <QtGui/QCursor>
 #include <QtGui/QFont>
 #include <QtGui/QPixmap>
 #include <QtGui/QFocusEvent>
@@ -96,6 +97,7 @@ ModelViewport::ModelViewport( QWidget * parent )
      m_rotY( 0.0 ),
      m_rotZ( 0.0 ),
      m_zoomLevel( 32.0 ),
+     m_unitWidth( 1.0 ),
      m_far( 10000.0 ),
      m_near( 1.0 ),
      m_farOrtho( 1000000.0 ),
@@ -106,7 +108,6 @@ ModelViewport::ModelViewport( QWidget * parent )
      m_overlayButton( ScrollButtonMAX ),
      m_capture( false ),
      m_texturesLoaded( false ),
-     m_viewOptions( ViewTexture ),
      m_toolbox( NULL )
 {
    // Default preferences
@@ -138,32 +139,22 @@ ModelViewport::ModelViewport( QWidget * parent )
    setAcceptDrops( true );
    setMouseTracking( true );
 
-   QPixmap arrow( arrow_xpm );
-   QPixmap cross( crosshairrow_xpm );
-
-   QImage img;
-
-   makeCurrent();
-
-   glGenTextures( 2, m_scrollTextures );
-
-   img = arrow.toImage();
-   makeTextureFromImage( img, m_scrollTextures[0] );
-
-   img = cross.toImage();
-   makeTextureFromImage( img, m_scrollTextures[1] );
-
    connect( m_scrollTimer, SIGNAL(timeout()), this, SLOT(scrollTimeout()));
+
+   setCursor( Qt::ArrowCursor );
 }
 
 ModelViewport::~ModelViewport()
 {
    log_debug( "deleting model viewport\n" );
 
-   makeCurrent();
+   if ( isValid() )
+   {
+      makeCurrent();
 
-   glDeleteTextures( 1, &m_backgroundTexture );
-   glDeleteTextures( 2, m_scrollTextures );
+      glDeleteTextures( 1, &m_backgroundTexture );
+      glDeleteTextures( 2, m_scrollTextures );
+   }
 
    freeTextures();
 
@@ -182,6 +173,25 @@ void ModelViewport::freeTextures()
 
 void ModelViewport::initializeGL()
 {
+   if ( !isValid() )
+   {
+      log_error( "model viewport does not have a valid OpenGL context\n" );
+      return;
+   }
+
+   QPixmap arrow( arrow_xpm );
+   QPixmap cross( crosshairrow_xpm );
+
+   QImage img;
+
+   glGenTextures( 2, m_scrollTextures );
+
+   img = arrow.toImage();
+   makeTextureFromImage( img, m_scrollTextures[0] );
+
+   img = cross.toImage();
+   makeTextureFromImage( img, m_scrollTextures[1] );
+
    glShadeModel( GL_SMOOTH );
    glClearColor( m_backColor.red() / 256.0, 
          m_backColor.green() / 256.0, 
@@ -252,6 +262,9 @@ void ModelViewport::resizeGL( int w, int h )
 
 void ModelViewport::paintGL()
 {
+   if ( !isValid() )
+      return;
+
    //LOG_PROFILE();
 
    if ( m_inOverlay )
@@ -378,7 +391,7 @@ void ModelViewport::paintGL()
             = (g_prefs( "ui_render_3d_selections" ).intValue() == 0) 
             ? false : true;
 
-         switch ( m_viewOptions )
+         switch ( m_model->getPerspectiveDrawMode() )
          {
             case ViewWireframe:
                opt = Model::DO_WIREFRAME;
@@ -399,23 +412,31 @@ void ModelViewport::paintGL()
 
          opt |= ( (g_prefs( "ui_render_backface_cull" ).intValue() == 0) ? 0 : Model::DO_BACKFACECULL);
 
+         if ( opt != Model::DO_WIREFRAME )
+         {
+            glEnable( GL_LIGHTING );
+
+            if ( drawSelections )
+            {
+               glEnable( GL_POLYGON_OFFSET_FILL );
+               glPolygonOffset( 1, 0 );
+            }
+
+            m_model->draw( opt, static_cast<ContextT>( this) , viewPoint );
+
+            if ( drawSelections )
+            {
+               glDisable( GL_POLYGON_OFFSET_FILL );
+               glPolygonOffset( 0, 0 );
+            }
+         }
+
          if ( drawSelections )
          {
             glDisable( GL_LIGHTING );
             m_model->drawLines();
             m_model->drawVertices();
-         }
 
-         if ( opt != Model::DO_WIREFRAME )
-         {
-            glEnable( GL_LIGHTING );
-            m_model->draw( opt, static_cast<ContextT>( this) , viewPoint );
-
-         }
-
-         if ( drawSelections )
-         {
-            glDisable( GL_LIGHTING );
             glDisable( GL_DEPTH_TEST );
             m_model->drawJoints();
          }
@@ -432,14 +453,13 @@ void ModelViewport::paintGL()
 
          glClear( GL_DEPTH_BUFFER_BIT );
 
-         m_model->drawLines();
-         m_model->drawVertices();
-
          int drawMode = m_model->getCanvasDrawMode();
          if ( drawMode != ViewWireframe )
          {
             glEnable( GL_LIGHTING );
             glEnable( GL_LIGHT0 );
+            glEnable( GL_POLYGON_OFFSET_FILL );
+            glPolygonOffset( 1, 0 );
 
             int opt = Model::DO_TEXTURE | Model::DO_SMOOTHING 
                | ( (g_prefs( "ui_render_bad_textures" ).intValue() == 0) ? 0 : Model::DO_BADTEX);
@@ -461,8 +481,13 @@ void ModelViewport::paintGL()
             m_model->draw( opt, static_cast<ContextT>( this ), viewPoint );
 
             glDisable( GL_LIGHTING );
+            glDisable( GL_POLYGON_OFFSET_FILL );
+            glPolygonOffset( 0, 0 );
          }
 
+         m_model->drawLines();
+         m_model->drawVertices();
+
          glDisable( GL_DEPTH_TEST );
          m_model->drawJoints();
          m_model->drawPoints();
@@ -560,18 +585,7 @@ void ModelViewport::drawGridLines()
    }
    else
    {
-      double maxDimension = (m_width > m_height) ? m_width : m_height;
-
-      double unitWidth = g_prefs( "ui_grid_inc" ).doubleValue();
-
-      while ( (maxDimension / unitWidth) > 16 )
-      {
-         unitWidth *= 2.0;
-      }
-      while ( (maxDimension / unitWidth) < 4 )
-      {
-         unitWidth /= 2.0;
-      }
+      m_unitWidth = getUnitWidth();
 
       double xRangeMin = 0;
       double xRangeMax = 0;
@@ -596,16 +610,16 @@ void ModelViewport::drawGridLines()
             yRangeMin = m_arcballPoint[1] - (m_height / 2.0);
             yRangeMax = m_arcballPoint[1] + (m_height / 2.0);
 
-            xStart = unitWidth * ((int) (xRangeMin / unitWidth));
-            yStart = unitWidth * ((int) (yRangeMin / unitWidth));
+            xStart = m_unitWidth * ((int) (xRangeMin / m_unitWidth));
+            yStart = m_unitWidth * ((int) (yRangeMin / m_unitWidth));
 
-            for ( x = xStart; x < xRangeMax; x += unitWidth )
+            for ( x = xStart; x < xRangeMax; x += m_unitWidth )
             {
                glVertex3f( x, yRangeMin, 0 );
                glVertex3f( x, yRangeMax, 0 );
             }
 
-            for ( y = yStart; y < yRangeMax; y += unitWidth )
+            for ( y = yStart; y < yRangeMax; y += m_unitWidth )
             {
                glVertex3f( xRangeMin, y, 0 );
                glVertex3f( xRangeMax, y, 0 );
@@ -618,16 +632,16 @@ void ModelViewport::drawGridLines()
             yRangeMin = m_arcballPoint[1] - (m_height / 2.0);
             yRangeMax = m_arcballPoint[1] + (m_height / 2.0);
 
-            xStart = unitWidth * ((int) (xRangeMin / unitWidth));
-            yStart = unitWidth * ((int) (yRangeMin / unitWidth));
+            xStart = m_unitWidth * ((int) (xRangeMin / m_unitWidth));
+            yStart = m_unitWidth * ((int) (yRangeMin / m_unitWidth));
 
-            for ( x = xStart; x < xRangeMax; x += unitWidth )
+            for ( x = xStart; x < xRangeMax; x += m_unitWidth )
             {
                glVertex3f( x, yRangeMin, 0 );
                glVertex3f( x, yRangeMax, 0 );
             }
 
-            for ( y = yStart; y < yRangeMax; y += unitWidth )
+            for ( y = yStart; y < yRangeMax; y += m_unitWidth )
             {
                glVertex3f( xRangeMin, y, 0 );
                glVertex3f( xRangeMax, y, 0 );
@@ -640,16 +654,16 @@ void ModelViewport::drawGridLines()
             yRangeMin = m_arcballPoint[1] - (m_height / 2.0);
             yRangeMax = m_arcballPoint[1] + (m_height / 2.0);
 
-            xStart = unitWidth * ((int) (xRangeMin / unitWidth));
-            yStart = unitWidth * ((int) (yRangeMin / unitWidth));
+            xStart = m_unitWidth * ((int) (xRangeMin / m_unitWidth));
+            yStart = m_unitWidth * ((int) (yRangeMin / m_unitWidth));
 
-            for ( x = xStart; x < xRangeMax; x += unitWidth )
+            for ( x = xStart; x < xRangeMax; x += m_unitWidth )
             {
                glVertex3f( 0, yRangeMin, x );
                glVertex3f( 0, yRangeMax, x );
             }
 
-            for ( y = yStart; y < yRangeMax; y += unitWidth )
+            for ( y = yStart; y < yRangeMax; y += m_unitWidth )
             {
                glVertex3f( 0, y, xRangeMin );
                glVertex3f( 0, y, xRangeMax );
@@ -662,16 +676,16 @@ void ModelViewport::drawGridLines()
             yRangeMin = m_arcballPoint[1] - (m_height / 2.0);
             yRangeMax = m_arcballPoint[1] + (m_height / 2.0);
 
-            xStart = unitWidth * ((int) (xRangeMin / unitWidth));
-            yStart = unitWidth * ((int) (yRangeMin / unitWidth));
+            xStart = m_unitWidth * ((int) (xRangeMin / m_unitWidth));
+            yStart = m_unitWidth * ((int) (yRangeMin / m_unitWidth));
 
-            for ( x = xStart; x < xRangeMax; x += unitWidth )
+            for ( x = xStart; x < xRangeMax; x += m_unitWidth )
             {
                glVertex3f( 0, yRangeMin, x );
                glVertex3f( 0, yRangeMax, x );
             }
 
-            for ( y = yStart; y < yRangeMax; y += unitWidth )
+            for ( y = yStart; y < yRangeMax; y += m_unitWidth )
             {
                glVertex3f( 0, y, xRangeMin );
                glVertex3f( 0, y, xRangeMax );
@@ -684,16 +698,16 @@ void ModelViewport::drawGridLines()
             yRangeMin = -m_arcballPoint[1] - (m_height / 2.0);
             yRangeMax = -m_arcballPoint[1] + (m_height / 2.0);
 
-            xStart = unitWidth * ((int) (xRangeMin / unitWidth));
-            yStart = unitWidth * ((int) (yRangeMin / unitWidth));
+            xStart = m_unitWidth * ((int) (xRangeMin / m_unitWidth));
+            yStart = m_unitWidth * ((int) (yRangeMin / m_unitWidth));
 
-            for ( x = xStart; x < xRangeMax; x += unitWidth )
+            for ( x = xStart; x < xRangeMax; x += m_unitWidth )
             {
                glVertex3f( x, 0, yRangeMin );
                glVertex3f( x, 0, yRangeMax );
             }
 
-            for ( y = yStart; y < yRangeMax; y += unitWidth )
+            for ( y = yStart; y < yRangeMax; y += m_unitWidth )
             {
                glVertex3f( xRangeMin, 0, y );
                glVertex3f( xRangeMax, 0, y );
@@ -706,16 +720,16 @@ void ModelViewport::drawGridLines()
             yRangeMin = m_arcballPoint[1] - (m_height / 2.0);
             yRangeMax = m_arcballPoint[1] + (m_height / 2.0);
 
-            xStart = unitWidth * ((int) (xRangeMin / unitWidth));
-            yStart = unitWidth * ((int) (yRangeMin / unitWidth));
+            xStart = m_unitWidth * ((int) (xRangeMin / m_unitWidth));
+            yStart = m_unitWidth * ((int) (yRangeMin / m_unitWidth));
 
-            for ( x = xStart; x < xRangeMax; x += unitWidth )
+            for ( x = xStart; x < xRangeMax; x += m_unitWidth )
             {
                glVertex3f( x, 0, yRangeMin );
                glVertex3f( x, 0, yRangeMax );
             }
 
-            for ( y = yStart; y < yRangeMax; y += unitWidth )
+            for ( y = yStart; y < yRangeMax; y += m_unitWidth )
             {
                glVertex3f( xRangeMin, 0, y );
                glVertex3f( xRangeMax, 0, y );
@@ -768,7 +782,7 @@ void ModelViewport::drawGridLines()
          // well). That is why it is disabled by default. You can set ui_render_text
          // to a non-zero value in your mm3drc file to enable text rendering.
          QString text;
-         text.sprintf( "%g", unitWidth );
+         text.sprintf( "%g", m_unitWidth );
          renderText( 2, this->height() - 12, text, QFont( "Sans", 10 ) );
       }
    }
@@ -1027,7 +1041,74 @@ void ModelViewport::makeTextureFromImage( const QImage & i, GLuint & t )
          GL_RGB, GL_UNSIGNED_BYTE,
          data );
 
-   delete data;
+   delete[] data;
+}
+
+double ModelViewport::getUnitWidth()
+{
+   if ( m_viewDirection == ViewPerspective || m_viewDirection == ViewOrtho )
+   {
+      return g_prefs( "ui_3dgrid_inc" ).doubleValue();
+   }
+
+   double unitWidth = g_prefs( "ui_grid_inc" ).doubleValue();
+
+   double ratio;
+   int scale_min, scale_max;
+
+   if ( g_prefs.exists( "ui_grid_mode" )
+         || !g_prefs.exists( "ui_grid_decimal" ) )
+   {
+      // Note early return for fixed width ('case 2:')
+      switch ( g_prefs( "ui_grid_mode" ).intValue() )
+      {
+         default:
+         case 0:  // Binary
+            ratio = 2.0;
+            scale_min = 4;
+            scale_max = 16;
+            break;
+         case 1:  // Decimal
+            ratio = 10.0;
+            scale_min = 2;
+            scale_max = 60;
+            break;
+         case 2:  // Fixed
+            return unitWidth;
+            break;
+      }
+   }
+   else
+   {
+      // ui_grid_decimal is set, and ui_grid_mode is not.
+
+      if ( g_prefs( "ui_grid_decimal" ).intValue() != 0 )
+      {
+         ratio = 10.0;
+         scale_min = 2;
+         scale_max = 60;
+         g_prefs( "ui_grid_mode" ) = 1;
+      }
+      else
+      {
+         ratio = 2.0;
+         scale_min = 4;
+         scale_max = 16;
+         g_prefs( "ui_grid_mode" ) = 0;
+      }
+   }
+
+   double maxDimension = (m_width > m_height) ? m_width : m_height;
+   while ( (maxDimension / unitWidth) > scale_max )
+   {
+      unitWidth *= ratio;
+   }
+   while ( (maxDimension / unitWidth) < scale_min )
+   {
+      unitWidth /= ratio;
+   }
+
+   return unitWidth;
 }
 
 void ModelViewport::updateBackground()
@@ -1040,7 +1121,7 @@ void ModelViewport::updateBackground()
 
       if ( m_backgroundFile[0] != '\0' )
       {
-         m_texture = TextureManager::getInstance()->getTexture( m_backgroundFile.c_str() );
+         m_texture = TextureManager::getInstance()->getTexture( m_backgroundFile.c_str(), false, false );
          if ( !m_texture )
          {
             QString str = tr("Could not load background %1").arg( m_backgroundFile.c_str() );
@@ -1069,6 +1150,9 @@ void ModelViewport::adjustViewport()
 
 void ModelViewport::setViewportDraw()
 {
+   if ( !isValid() )
+      return;
+
    makeCurrent();
 
    m_inOverlay = false;
@@ -1245,6 +1329,11 @@ void ModelViewport::zoomIn()
          m_zoomLevel *= (VP_ZOOMSCALE);
       }
 
+      m_unitWidth = getUnitWidth();
+      char str[80];
+      PORT_snprintf( str, sizeof(str), "Units: %g", m_unitWidth );
+      model_status( m_model, StatusNormal, STATUSTIME_NONE, str );
+
       QString zoomStr;
       zoomStr.sprintf( "%f", m_zoomLevel );
       emit zoomLevelChanged( zoomStr );
@@ -1263,6 +1352,11 @@ void ModelViewport::zoomOut()
          m_zoomLevel /= VP_ZOOMSCALE;
       }
 
+      m_unitWidth = getUnitWidth();
+      char str[80];
+      PORT_snprintf( str, sizeof(str), "Units: %g", m_unitWidth );
+      model_status( m_model, StatusNormal, STATUSTIME_NONE, str );
+
       QString zoomStr;
       zoomStr.sprintf( "%f", m_zoomLevel );
       emit zoomLevelChanged( zoomStr );
@@ -1560,12 +1654,14 @@ void ModelViewport::mouseMoveEvent( QMouseEvent * e )
       }
 
       char str[80];
-      PORT_snprintf( str, sizeof(str), "%g, %g, %g", pos[0], pos[1], pos[2] );
+      PORT_snprintf( str, sizeof(str), "Units: %g  (%g, %g, %g)",
+            m_unitWidth, pos[0], pos[1], pos[2] );
       model_status( m_model, StatusNormal, STATUSTIME_NONE, str );
    }
    else
    {
-      model_status( m_model, StatusNormal, STATUSTIME_NONE, "" );
+      model_status( m_model, StatusNormal, STATUSTIME_NONE, "Units: %g",
+            getUnitWidth());
    }
 
    if ( m_operation == MO_Rotate )
@@ -2202,36 +2298,6 @@ void ModelViewport::setZoomLevel( double zoom )
    }
 }
 
-void ModelViewport::wireframeEvent()
-{
-   m_viewOptions = ViewWireframe;
-   updateGL();
-}
-
-void ModelViewport::flatEvent()
-{
-   m_viewOptions = ViewFlat;
-   updateGL();
-}
-
-void ModelViewport::smoothEvent()
-{
-   m_viewOptions = ViewSmooth;
-   updateGL();
-}
-
-void ModelViewport::textureEvent()
-{
-   m_viewOptions = ViewTexture;
-   updateGL();
-}
-
-void ModelViewport::alphaEvent()
-{
-   m_viewOptions = ViewAlpha;
-   updateGL();
-}
-
 void ModelViewport::scrollTimeout()
 {
    if ( m_operation == MO_RotateButton )
@@ -2283,6 +2349,8 @@ void ModelViewport::scrollTimeout()
 
 void ModelViewport::focusInEvent( QFocusEvent * e )
 {
+   if ( !isValid() )
+      return;
    m_backColor.setRgb( 150, 210, 210 );
    makeCurrent();
    glClearColor( m_backColor.red() / 256.0, 
@@ -2293,6 +2361,8 @@ void ModelViewport::focusInEvent( QFocusEvent * e )
 
 void ModelViewport::focusOutEvent( QFocusEvent * e )
 {
+   if ( !isValid() )
+      return;
    m_backColor.setRgb( 130, 200, 200 );
    makeCurrent();
    glClearColor( m_backColor.red() / 256.0, 
@@ -2471,20 +2541,6 @@ void ModelViewport::getParentXYValue( int x, int y, double & xval, double & yval
    {
       // snap to grid
 
-      double unitWidth = 1.0;
-      double maxDimension = (m_width > m_height) ? m_width : m_height;
-
-      unitWidth = g_prefs( "ui_grid_inc" ).doubleValue();
-
-      while ( (maxDimension / unitWidth) > 16 )
-      {
-         unitWidth *= 2.0;
-      }
-      while ( (maxDimension / unitWidth) < 4 )
-      {
-         unitWidth /= 2.0;
-      }
-
       double val;
       int mult;
       double fudge;
@@ -2502,8 +2558,8 @@ void ModelViewport::getParentXYValue( int x, int y, double & xval, double & yval
          fudge = -0.5;
       }
 
-      mult = (int) (x / unitWidth + fudge);
-      val = (double) mult * unitWidth;
+      mult = (int) (x / m_unitWidth + fudge);
+      val = (double) mult * m_unitWidth;
 
       if ( fabs( x - val ) < maxDist )
       {
@@ -2527,8 +2583,8 @@ void ModelViewport::getParentXYValue( int x, int y, double & xval, double & yval
          fudge = -0.5;
       }
 
-      mult = (int) (y / unitWidth + fudge);
-      val = (double) mult * unitWidth;
+      mult = (int) (y / m_unitWidth + fudge);
+      val = (double) mult * m_unitWidth;
 
       if ( fabs( y - val ) < maxDist )
       {
@@ -3014,6 +3070,9 @@ void ModelViewport::checkGlErrors()
 
 void ModelViewport::copyContentsToTexture( Texture * tex )
 {
+   if ( !isValid() )
+      return;
+
    makeCurrent();
 
    m_capture = true;
