@@ -1,4 +1,4 @@
-/*  Misfit Model 3D
+/*  Maverick Model 3D
  * 
  *  Copyright (c) 2004-2007 Kevin Worcester
  * 
@@ -165,12 +165,6 @@ class ModelViewport : public QGLWidget, public Tool::Parent
       void setZoomLevel( double zoomLevel );
       void setViewState( const ModelViewport::ViewStateT & viewState );
 
-      void wireframeEvent();
-      void flatEvent();
-      void smoothEvent();
-      void textureEvent();
-      void alphaEvent();
-
       void scrollTimeout();
 
    protected slots:
@@ -203,6 +197,8 @@ class ModelViewport : public QGLWidget, public Tool::Parent
 
       void makeTextureFromImage( const QImage & i, GLuint & t );
 
+      double getUnitWidth();
+
       Model * m_model;
 
       MouseOperationE m_operation;
@@ -223,6 +219,7 @@ class ModelViewport : public QGLWidget, public Tool::Parent
       double m_width;
       double m_height;
       double m_zoomLevel;
+      double m_unitWidth;
       double m_far;
       double m_near;
       double m_farOrtho;
