@@ -1,4 +1,4 @@
-/*  Misfit Model 3D
+/*  Maverick Model 3D
  * 
  *  Copyright (c) 2004-2007 Kevin Worcester
  * 
@@ -27,9 +27,11 @@
 #include "sysconf.h"
 #include "config.h"
 
+#include <QtCore/QtGlobal>
 #include <QtCore/QLocale>
 #include <QtCore/QTranslator>
-#include <QtGui/QApplication>
+#include <QtWidgets/QApplication>
+#include <QtGui/QFileOpenEvent>
 #include <QtOpenGL/QGLFormat>
 
 #include <list>
@@ -47,7 +49,29 @@
 
 #include "mlocale.h"
 
-static QApplication * s_app = NULL;
+// Handle macOS opening file in finder or dropping file on dock icon.
+// File types must be listed in AppBundle.app/Contents/Info.plist.
+class ModelApp : public QApplication
+{
+public:
+   ModelApp(int &argc, char **argv)
+       : QApplication(argc, argv)
+   {
+   }
+
+   bool event(QEvent *event)
+   {
+      if ( event->type() == QEvent::FileOpen ) {
+         QFileOpenEvent *openEvent = static_cast<QFileOpenEvent *>(event);
+
+         ViewWindow::openModelInEmptyWindow( openEvent->file().toUtf8() );
+      }
+
+      return QApplication::event(event);
+   }
+};
+
+static ModelApp     * s_app = NULL;
 static QTranslator  * s_qtXlat = NULL;
 static QTranslator  * s_mm3dXlat = NULL;
 
@@ -109,7 +133,20 @@ QApplication * ui_getapp()
 
 int ui_prep( int & argc, char * argv[] )
 {
-   s_app = new QApplication( argc, argv );
+   s_app = new ModelApp( argc, argv );
+
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 10, 0))
+   // Disable "?" button (What's this?) on Windows QDialogs
+   s_app->setAttribute( Qt::AA_DisableWindowContextHelpButton );
+#endif
+
+#ifdef Q_OS_MAC
+   // Don't use native mac menu bar as it's misleading (the menu affects
+   // minimized window when it should not). This could be fixed by using
+   // a separate menu when no window is active but I've failed to get
+   // that to work correct with Qt 5.11.1.
+   s_app->setAttribute( Qt::AA_DontUseNativeMenuBar );
+#endif
 
    QString loc = mlocale_get().c_str();
 
@@ -162,10 +199,9 @@ int ui_init( int & argc, char * argv[] )
 
       if ( openCount == 0 )
       {
+         char * pwd = ( argc > 1 ) ? PORT_get_current_dir_name() : NULL;
          for ( int t = 1; t < argc; t++ )
          {
-            char pwd[ PATH_MAX ];
-            getcwd( pwd, PATH_MAX );
             std::string file = normalizePath( argv[t], pwd );
             if ( ViewWindow::openModel( file.c_str() ) )
             {
@@ -173,6 +209,10 @@ int ui_init( int & argc, char * argv[] )
                opened = true;
             }
          }
+         if ( pwd )
+         {
+            free( pwd );
+         }
       }
       else
       {
