@@ -1,4 +1,4 @@
-/*  Misfit Model 3D
+/*  Maverick Model 3D
  * 
  *  Copyright (c) 2004-2007 Kevin Worcester
  * 
@@ -24,14 +24,12 @@
 #ifndef __QTMAIN_H
 #define __QTMAIN_H
 
-#include "mq3compat.h"
-
 class QApplication;
 
 extern QApplication * ui_getapp();
 
-extern int ui_prep( int argc, char * argv[] );
-extern int ui_init( int argc, char * argv[] );
+extern int ui_prep( int & argc, char * argv[] );
+extern int ui_init( int & argc, char * argv[] );
 extern void ui_exit();
 
 #endif // __QTMAIN_H
