@@ -1,4 +1,4 @@
-/*  Misfit Model 3D
+/*  Maverick Model 3D
  * 
  *  Copyright (c) 2004-2007 Kevin Worcester
  * 
@@ -28,7 +28,7 @@
 
 #include "model.h"
 
-#include <QtGui/QDialog>
+#include <QtWidgets/QDialog>
 
 #include <list>
 #include <map>
@@ -64,6 +64,7 @@ class TextureCoord : public QDialog, public Ui::TextureCoordBase, public Model::
 
    public slots:
       void show();
+      void closeEvent( QCloseEvent * e );
       void helpNowEvent();
       void toolSelectEvent();
       void toolMoveEvent();
@@ -76,8 +77,17 @@ class TextureCoord : public QDialog, public Ui::TextureCoordBase, public Model::
       virtual void scaleSettingsChangedEvent();
 
       virtual void updateTextureCoordsEvent();
+      virtual void updateSelectionDoneEvent();
       virtual void updateDoneEvent();
 
+      void rotateCcwEvent();
+      void rotateCwEvent();
+      void vFlipEvent();
+      void hFlipEvent();
+
+      void selectionColorChangedEvent(int);
+      void linesColorChangedEvent(int);
+
       void zoomIn();
       void zoomOut();
       void undoEvent();
