@@ -1,4 +1,4 @@
-/*  Misfit Model 3D
+/*  Maverick Model 3D
  * 
  *  Copyright (c) 2004-2007 Kevin Worcester
  * 
@@ -36,14 +36,14 @@
 #include "helpwin.h"
 #include "errorobj.h"
 
-#include <QtGui/QComboBox>
-#include <QtGui/QPushButton>
-#include <QtGui/QInputDialog>
-#include <QtGui/QFileDialog>
-#include <QtGui/QLineEdit>
-#include <QtGui/QSlider>
-#include <QtGui/QLabel>
-#include <QtGui/QShortcut>
+#include <QtWidgets/QComboBox>
+#include <QtWidgets/QPushButton>
+#include <QtWidgets/QInputDialog>
+#include <QtWidgets/QFileDialog>
+#include <QtWidgets/QLineEdit>
+#include <QtWidgets/QSlider>
+#include <QtWidgets/QLabel>
+#include <QtWidgets/QShortcut>
 
 #include <list>
 #include <string>
@@ -152,7 +152,7 @@ void TextureWindow::changeTextureFileEvent()
    QFileDialog d(NULL, "", dir, formatsStr + QString(";; All Files (*)" ) );
 
    d.setWindowTitle( tr("Open texture image") );
-   d.selectFilter( formatsStr );
+   d.selectNameFilter( formatsStr );
 
    int execval = d.exec();
    QStringList files = d.selectedFiles();
@@ -200,7 +200,7 @@ void TextureWindow::noTextureFileEvent()
 void TextureWindow::newMaterialClickedEvent()
 {
    bool ok = false;
-   QString name = QInputDialog::getText( this, tr( "Color Material", "window title" ), tr( "Enter new material name:" ), QLineEdit::Normal, QString(""), &ok );
+   QString name = QInputDialog::getText( this, tr( "New Material", "window title" ), tr( "Enter new material name:" ), QLineEdit::Normal, QString(""), &ok );
 
    if ( ok )
    {
@@ -219,7 +219,7 @@ void TextureWindow::renameClickedEvent()
    if ( textureId >= 0 )
    {
       bool ok = false;
-      QString name = QInputDialog::getText( this, tr( "Rename texture", "window title" ), tr( "Enter new texture name:" ), QLineEdit::Normal, QString::fromUtf8( m_model->getTextureName( textureId ) ), &ok );
+      QString name = QInputDialog::getText( this, tr( "Rename material", "window title" ), tr( "Enter new material name:" ), QLineEdit::Normal, QString::fromUtf8( m_model->getTextureName( textureId ) ), &ok );
 
       if ( ok )
       {
@@ -542,7 +542,7 @@ void TextureWindow::alphaSliderChanged( int v )
 void TextureWindow::redEditChanged( const QString & str )
 {
    m_editing = true;
-   float v = atof( str.toLatin1() );
+   float v = str.toDouble();
    m_redSlider->setValue( (int) (v * 100) );
    m_editing = false;
 }
@@ -550,7 +550,7 @@ void TextureWindow::redEditChanged( const QString & str )
 void TextureWindow::greenEditChanged( const QString & str )
 {
    m_editing = true;
-   float v = atof( str.toLatin1() );
+   float v = str.toDouble();
    m_greenSlider->setValue( (int) (v * 100) );
    m_editing = false;
 }
@@ -558,7 +558,7 @@ void TextureWindow::greenEditChanged( const QString & str )
 void TextureWindow::blueEditChanged( const QString & str )
 {
    m_editing = true;
-   float v = atof( str.toLatin1() );
+   float v = str.toDouble();
    m_blueSlider->setValue( (int) (v * 100) );
    m_editing = false;
 }
@@ -566,7 +566,7 @@ void TextureWindow::blueEditChanged( const QString & str )
 void TextureWindow::alphaEditChanged( const QString & str )
 {
    m_editing = true;
-   float v = atof( str.toLatin1() );
+   float v = str.toDouble();
    m_alphaSlider->setValue( (int) (v * 100) );
    m_editing = false;
 }
