@@ -1,4 +1,4 @@
-/*  Misfit Model 3D
+/*  Maverick Model 3D
  * 
  *  Copyright (c) 2004-2007 Kevin Worcester
  * 
@@ -25,15 +25,15 @@
 #include "helpwin.h"
 #include "log.h"
 
-#include <QtGui/QSlider>
-#include <QtGui/QLineEdit>
-#include <QtGui/QLabel>
-#include <QtGui/QShortcut>
+#include <QtWidgets/QSlider>
+#include <QtWidgets/QLineEdit>
+#include <QtWidgets/QLabel>
+#include <QtWidgets/QShortcut>
 
 #include <math.h>
 #include <stdlib.h>
 
-ValueWin::ValueWin( QWidget * parent, bool modal, Qt::WFlags flags )
+ValueWin::ValueWin( QWidget * parent, bool modal, Qt::WindowFlags flags )
    : QDialog( parent, flags ),
      m_editing( false )
 {
@@ -94,7 +94,7 @@ void ValueWin::valueSliderChanged( int v )
 void ValueWin::valueEditChanged( const QString & str )
 {
    m_editing = true;
-   float v = atof( str.toLatin1() );
+   float v = str.toDouble();
    m_valueSlider->setValue( (int) v );
    m_editing = false;
 }
