@@ -1,4 +1,4 @@
-/*  Misfit Model 3D
+/*  Maverick Model 3D
  * 
  *  Copyright (c) 2004-2007 Kevin Worcester
  * 
@@ -42,18 +42,6 @@
 #include <stdint.h>
 #include <ctype.h>
 
-// FIXME this should be centralized
-template<typename T>
-class FunctionCaller
-{
-   public:
-      FunctionCaller( T * obj, void (T::*method)(void) ) { m_obj = obj; m_method = method; }
-      ~FunctionCaller() { (m_obj->*m_method)(); }
-   private:
-      T * m_obj;
-      void (T::*m_method)(void);
-};
-
 const unsigned MAX_QUAKE_NORMALS = 162;
 static float s_quakeNormals[ MAX_QUAKE_NORMALS ][3] = {
 #include "md2filter-anorms.h"
@@ -124,7 +112,7 @@ Model::ModelErrorE Md2Filter::readFile( Model * model, const char * const filena
    {
       Model::ModelErrorE err = Model::ERROR_NONE;
       DataSource* src = openInput( filename, err );
-      FunctionCaller<DataSource> fc( src, &DataSource::close );
+      SourceCloser fc( src );
 
       if ( err != Model::ERROR_NONE )
          return err;
@@ -347,6 +335,7 @@ Model::ModelErrorE Md2Filter::readFile( Model * model, const char * const filena
       for ( i = 0; i < numSkins; i++ )
       {
          src->readBytes( (uint8_t *) name, 64 );
+         name[63] = '\0';
          log_debug( "Skin %d: %s\n", i, name );
 
          string tempStr = name;
@@ -590,7 +579,7 @@ Model::ModelErrorE Md2Filter::writeFile( Model * model, const char * const filen
 
       Model::ModelErrorE err = Model::ERROR_NONE;
       DataDest * dst = openOutput( filename, err );
-      FunctionCaller<DataDest> fc( dst, &DataDest::close );
+      DestCloser fc( dst );
 
       if ( err != Model::ERROR_NONE )
          return err;
@@ -922,9 +911,8 @@ Model::ModelErrorE Md2Filter::writeFile( Model * model, const char * const filen
                   float    s1 = stripList[t][i].s;
                   float    t1 = 1.0f - stripList[t][i].t;
 
-                  // FIXME this is reversed from below. Is it correct?
-                  dst->write(t1);
                   dst->write(s1);
+                  dst->write(t1);
                   dst->write(v1);
                }
             }
@@ -941,10 +929,9 @@ Model::ModelErrorE Md2Filter::writeFile( Model * model, const char * const filen
                   float    s1 = fanList[t][i].s;
                   float    t1 = 1.0f - fanList[t][i].t;
 
-                  // FIXME this is reversed from above. Is it correct?
-                  dst->write(v1);
                   dst->write(s1);
                   dst->write(t1);
+                  dst->write(v1);
                }
             }
          }
