@@ -1,4 +1,4 @@
-/*  Misfit Model 3D
+/*  Maverick Model 3D
  * 
  *  Copyright (c) 2004-2007 Kevin Worcester
  * 
@@ -183,6 +183,7 @@ class Model
          PropDimensions  = 0x100000,  // 
          PropPixels      = 0x200000,  // 
          PropTime        = 0x400000,  // 
+         PropLoop        = 0x800000,  // 
          PropAll         = 0xFFFFFF,  // 
 
          // These are combinations of parts above, for convenience
@@ -254,6 +255,13 @@ class Model
                return m_boneId < rhs.m_boneId;
             return m_weight < rhs.m_weight;
          }
+
+         bool operator>( const struct _Influence_t & rhs ) const
+         {
+            if ( fabs( m_weight - rhs.m_weight ) < 0.00001 )
+               return m_boneId > rhs.m_boneId;
+            return m_weight > rhs.m_weight;
+         }
       };
       typedef struct _Influence_t InfluenceT;
       typedef std::list< InfluenceT > InfluenceList;
@@ -651,6 +659,7 @@ class Model
             JointKeyframeList m_jointKeyframes;
             double   m_fps;  // Frames per second
             double   m_spf;  // Seconds per frame (for convenience, 1.0 / m_fps)
+            bool     m_loop; // Whether or not the animation loops
             unsigned m_frameCount;    // Number of frames in the animation
             bool     m_validNormals;  // Whether or not the normals have been calculated for the current animation frame
 
@@ -766,6 +775,7 @@ class Model
             FrameAnimDataList m_frameData;
 
             double m_fps;  // Frames per second
+            bool   m_loop; // Whether or not the animation loops
             bool   m_validNormals;  // Whether or not the normals have been calculated
 
             bool propEqual( const FrameAnim & rhs, int propBits = PropAll, double tolerance = 0.00001 ) const;
@@ -1047,7 +1057,7 @@ class Model
       void pushError( const std::string & err );
       std::string popError();
 
-      // Use these functions to preserve data that Misfit doesn't support natively
+      // Use these functions to preserve data that MM3D doesn't support natively
       int  addFormatData( FormatData * fd );
       bool deleteFormatData( unsigned index );
       unsigned getFormatDataCount() const;
@@ -1109,9 +1119,6 @@ class Model
       unsigned getCurrentAnimationFrame() const;
       double   getCurrentAnimationTime() const;
 
-      void setAnimationLooping( bool o );
-      bool isAnimationLooping() const;
-
       // Stop animation mode, go back to standard pose editing.
       void setNoAnimation();
 
@@ -1130,6 +1137,9 @@ class Model
       double getAnimFPS( AnimationModeE mode, unsigned anim ) const;
       bool setAnimFPS( AnimationModeE mode, unsigned anim, double fps );
 
+      bool getAnimLooping( AnimationModeE mode, unsigned anim ) const;
+      bool setAnimLooping( AnimationModeE mode, unsigned anim, bool loop );
+
       unsigned getAnimFrameCount( AnimationModeE mode, unsigned anim ) const;
       bool setAnimFrameCount( AnimationModeE mode, unsigned anim, unsigned count );
 
@@ -1214,8 +1224,6 @@ class Model
       // the model has frame animations.
       void displayFrameAnimPrimitiveError();
 
-      int getNumFrames() const;  // Deprecated
-
       // ------------------------------------------------------------------
       // Normal functions
       // ------------------------------------------------------------------
@@ -1358,7 +1366,6 @@ class Model
 
       const char * getTextureFilename( unsigned textureId ) const;
       Texture * getTextureData( unsigned textureId );
-      const Texture * getTextureData( unsigned textureId ) const { return getTextureData( textureId ); }
 
       // Lighting accessors
       bool getTextureAmbient(   unsigned textureId,       float * ambient   ) const;
@@ -1411,7 +1418,8 @@ class Model
       bool getBoneJointFinalMatrix( unsigned jointNumber, Matrix & m ) const;
       bool getBoneJointAbsoluteMatrix( unsigned jointNumber, Matrix & m ) const;
       bool getBoneJointRelativeMatrix( unsigned jointNumber, Matrix & m ) const;
-      bool getPointFinalMatrix( unsigned jointNumber, Matrix & m ) const;
+      bool getPointFinalMatrix( unsigned pointNumber, Matrix & m ) const;
+      bool getPointAbsoluteMatrix( unsigned pointNumber, Matrix & m ) const;
 
       list<int> getBoneJointVertices( int joint ) const;
       int getVertexBoneJoint( unsigned vertexNumber ) const;
@@ -1865,10 +1873,7 @@ class Model
 
       bool m_forceAddOrDelete;
 
-      int    m_numFrames;  // Deprecated
-
       AnimationModeE m_animationMode;
-      bool     m_animationLoop;
       unsigned m_currentFrame;
       unsigned m_currentAnim;
       double   m_currentTime;
