@@ -1,4 +1,4 @@
-/*  Misfit Model 3D
+/*  Maverick Model 3D
  * 
  *  Copyright (c) 2004-2007 Kevin Worcester
  * 
@@ -33,7 +33,7 @@
 
 #ifdef MM3D_EDIT
 
-int Model::addAnimation( const AnimationModeE & m, const char * name )
+int Model::addAnimation( AnimationModeE m, const char * name )
 {
    LOG_PROFILE();
 
@@ -50,6 +50,7 @@ int Model::addAnimation( const AnimationModeE & m, const char * name )
                anim->m_name = name;
                anim->m_fps  = 30.0;
                anim->m_spf  = (1.0 / anim->m_fps);
+               anim->m_loop = true;
                anim->m_frameCount = 1;
                anim->m_validNormals = false;
 
@@ -73,6 +74,7 @@ int Model::addAnimation( const AnimationModeE & m, const char * name )
                FrameAnim * anim = FrameAnim::get();
                anim->m_name = name;
                anim->m_fps = 10.0;
+               anim->m_loop = true;
                anim->m_validNormals = false;
 
                MU_AddAnimation * undo = new MU_AddAnimation();
@@ -90,7 +92,7 @@ int Model::addAnimation( const AnimationModeE & m, const char * name )
    return num;
 }
 
-void Model::deleteAnimation( const AnimationModeE & m, const unsigned & index )
+void Model::deleteAnimation( AnimationModeE m, unsigned index )
 {
    LOG_PROFILE();
 
@@ -121,7 +123,7 @@ void Model::deleteAnimation( const AnimationModeE & m, const unsigned & index )
    }
 }
 
-bool Model::setAnimName( const AnimationModeE & m, const unsigned & anim, const char * name )
+bool Model::setAnimName( AnimationModeE m, unsigned anim, const char * name )
 {
    switch ( m )
    {
@@ -129,7 +131,7 @@ bool Model::setAnimName( const AnimationModeE & m, const unsigned & anim, const
          if ( anim < m_skelAnims.size() )
          {
             MU_SetAnimName * undo = new MU_SetAnimName();
-            undo->setName( m_animationMode, anim, name, m_skelAnims[anim]->m_name.c_str() );
+            undo->setName( m, anim, name, m_skelAnims[anim]->m_name.c_str() );
             sendUndo( undo );
 
             m_skelAnims[ anim ]->m_name = name;
@@ -140,7 +142,7 @@ bool Model::setAnimName( const AnimationModeE & m, const unsigned & anim, const
          if ( anim < m_frameAnims.size() )
          {
             MU_SetAnimName * undo = new MU_SetAnimName();
-            undo->setName( m_animationMode, anim, name, m_frameAnims[anim]->m_name.c_str() );
+            undo->setName( m, anim, name, m_frameAnims[anim]->m_name.c_str() );
             sendUndo( undo );
 
             m_frameAnims[ anim ]->m_name = name;
@@ -153,7 +155,7 @@ bool Model::setAnimName( const AnimationModeE & m, const unsigned & anim, const
    return false;
 }
 
-bool Model::setAnimFrameCount( const AnimationModeE & m, const unsigned & anim, const unsigned & count )
+bool Model::setAnimFrameCount( AnimationModeE m, unsigned anim, unsigned count )
 {
    switch ( m )
    {
@@ -266,7 +268,7 @@ bool Model::setAnimFrameCount( const AnimationModeE & m, const unsigned & anim,
    return false;
 }
 
-bool Model::setAnimFPS( const AnimationModeE & m, const unsigned & anim, const double & fps )
+bool Model::setAnimFPS( AnimationModeE m, unsigned anim, double fps )
 {
    switch ( m )
    {
@@ -310,7 +312,40 @@ bool Model::setAnimFPS( const AnimationModeE & m, const unsigned & anim, const d
    return false;
 }
 
-void Model::setFrameAnimPointCount( const unsigned & pointCount )
+bool Model::setAnimLooping( AnimationModeE m, unsigned anim, bool loop )
+{
+   switch ( m )
+   {
+      case ANIMMODE_SKELETAL:
+         if ( anim < m_skelAnims.size() )
+         {
+            MU_SetAnimLoop * undo = new MU_SetAnimLoop();
+            undo->setAnimLoop( m, anim, loop, m_skelAnims[anim]->m_loop );
+            sendUndo( undo );
+
+            m_skelAnims[anim]->m_loop = loop;
+            return true;
+         }
+         break;
+      case ANIMMODE_FRAME:
+         if ( anim < m_frameAnims.size() )
+         {
+            MU_SetAnimLoop * undo = new MU_SetAnimLoop();
+            undo->setAnimLoop( m, anim, loop, m_frameAnims[anim]->m_loop );
+            sendUndo( undo );
+
+            m_frameAnims[anim]->m_loop = loop;
+            return true;
+         }
+         break;
+      default:
+         break;
+   }
+
+   return false;
+}
+
+void Model::setFrameAnimPointCount( unsigned pointCount )
 {
    unsigned anim = 0;
    unsigned frame = 0;
@@ -346,8 +381,8 @@ void Model::setFrameAnimPointCount( const unsigned & pointCount )
    sendUndo( undo );
 }
 
-bool Model::setFrameAnimPointCoords( const unsigned & anim, const unsigned & frame, const unsigned & point,
-      const double & x, const double & y, const double & z )
+bool Model::setFrameAnimPointCoords( unsigned anim, unsigned frame, unsigned point,
+      double x, double y, double z )
 {
    if ( anim < m_frameAnims.size() 
          && frame < m_frameAnims[anim]->m_frameData.size()
@@ -380,8 +415,8 @@ bool Model::setFrameAnimPointCoords( const unsigned & anim, const unsigned & fra
    }
 }
 
-bool Model::setFrameAnimPointRotation( const unsigned & anim, const unsigned & frame, const unsigned & point,
-      const double & x, const double & y, const double & z )
+bool Model::setFrameAnimPointRotation( unsigned anim, unsigned frame, unsigned point,
+      double x, double y, double z )
 {
    if ( anim < m_frameAnims.size() 
          && frame < m_frameAnims[anim]->m_frameData.size()
@@ -414,8 +449,8 @@ bool Model::setFrameAnimPointRotation( const unsigned & anim, const unsigned & f
    }
 }
 
-bool Model::getFrameAnimPointCoords( const unsigned & anim, const unsigned & frame, const unsigned & point,
-      double & x, double & y, double & z )
+bool Model::getFrameAnimPointCoords( unsigned anim, unsigned frame, unsigned point,
+      double & x, double & y, double & z ) const
 {
    if ( anim < m_frameAnims.size() 
          && frame < m_frameAnims[anim]->m_frameData.size()
@@ -440,8 +475,8 @@ bool Model::getFrameAnimPointCoords( const unsigned & anim, const unsigned & fra
    }
 }
 
-bool Model::getFrameAnimPointRotation( const unsigned & anim, const unsigned & frame, const unsigned & point,
-      double & x, double & y, double & z )
+bool Model::getFrameAnimPointRotation( unsigned anim, unsigned frame, unsigned point,
+      double & x, double & y, double & z ) const
 {
    if ( anim < m_frameAnims.size() 
          && frame < m_frameAnims[anim]->m_frameData.size()
@@ -466,7 +501,7 @@ bool Model::getFrameAnimPointRotation( const unsigned & anim, const unsigned & f
    }
 }
 
-void Model::setFrameAnimVertexCount( const unsigned & vertexCount )
+void Model::setFrameAnimVertexCount( unsigned vertexCount )
 {
    unsigned anim = 0;
    unsigned frame = 0;
@@ -502,8 +537,8 @@ void Model::setFrameAnimVertexCount( const unsigned & vertexCount )
    sendUndo( undo );
 }
 
-bool Model::setFrameAnimVertexCoords( const unsigned & anim, const unsigned & frame, const unsigned & vertex,
-      const double & x, const double & y, const double & z )
+bool Model::setFrameAnimVertexCoords( unsigned anim, unsigned frame, unsigned vertex,
+      double x, double y, double z )
 {
    if ( anim < m_frameAnims.size() 
          && frame < m_frameAnims[anim]->m_frameData.size()
@@ -541,7 +576,35 @@ bool Model::setFrameAnimVertexCoords( const unsigned & anim, const unsigned & fr
    }
 }
 
-int Model::copyAnimation( const AnimationModeE & mode, const unsigned & anim, const char * newName )
+bool Model::setQuickFrameAnimVertexCoords( unsigned anim, unsigned frame, unsigned vertex,
+      double x, double y, double z )
+{
+   if ( anim < m_frameAnims.size() 
+         && frame < m_frameAnims[anim]->m_frameData.size()
+         && vertex < m_vertices.size() )
+   {
+      if ( vertex >= m_frameAnims[anim]->m_frameData[frame]->m_frameVertices->size() )
+      {
+         log_warning( "resize the animation vertex list before calling setQuickFrameAnimVertexCoords\n" );
+         setFrameAnimVertexCount( vertex );
+      }
+
+      FrameAnimVertex * fav = (*m_frameAnims[anim]->m_frameData[frame]->m_frameVertices)[vertex];
+
+      fav->m_coord[0] = x;
+      fav->m_coord[1] = y;
+      fav->m_coord[2] = z;
+
+      m_frameAnims[anim]->m_validNormals = false;
+      return true;
+   }
+   else
+   {
+      return false;
+   }
+}
+
+int Model::copyAnimation( AnimationModeE mode, unsigned anim, const char * newName )
 {
    int num = -1;
    switch ( mode )
@@ -554,6 +617,7 @@ int Model::copyAnimation( const AnimationModeE & mode, const unsigned & anim, co
             {
                setAnimFrameCount( mode, num, getAnimFrameCount( mode, anim ) );
                setAnimFPS( mode, num, getAnimFPS( mode, anim ) );
+               setAnimLooping( mode, num, getAnimLooping( mode, anim ) );
 
                SkelAnim * sa = m_skelAnims[anim];
 
@@ -578,6 +642,7 @@ int Model::copyAnimation( const AnimationModeE & mode, const unsigned & anim, co
             {
                setAnimFrameCount( mode, num, getAnimFrameCount( mode, anim ) );
                setAnimFPS( mode, num, getAnimFPS( mode, anim ) );
+               setAnimLooping( mode, num, getAnimLooping( mode, anim ) );
 
                FrameAnim * fa = m_frameAnims[anim];
 
@@ -608,7 +673,7 @@ int Model::copyAnimation( const AnimationModeE & mode, const unsigned & anim, co
    return num;
 }
 
-int Model::splitAnimation( const AnimationModeE & mode, const unsigned & anim, const char * newName, const unsigned & frame )
+int Model::splitAnimation( AnimationModeE mode, unsigned anim, const char * newName, unsigned frame )
 {
    int num = -1;
    switch ( mode )
@@ -684,7 +749,7 @@ int Model::splitAnimation( const AnimationModeE & mode, const unsigned & anim, c
    return num;
 }
 
-bool Model::joinAnimations( const AnimationModeE & mode, const unsigned & anim1, const unsigned & anim2 )
+bool Model::joinAnimations( AnimationModeE mode, unsigned anim1, unsigned anim2 )
 {
    log_debug( "join %d anim %d + %d\n", mode, anim1, anim2 );
    if ( anim1 == anim2 )
@@ -759,7 +824,7 @@ bool Model::joinAnimations( const AnimationModeE & mode, const unsigned & anim1,
    return false;
 }
 
-bool Model::mergeAnimations( const AnimationModeE & mode, const unsigned & anim1, const unsigned & anim2 )
+bool Model::mergeAnimations( AnimationModeE mode, unsigned anim1, unsigned anim2 )
 {
    log_debug( "merge %d anim %d + %d\n", mode, anim1, anim2 );
    if ( anim1 == anim2 )
@@ -803,7 +868,7 @@ bool Model::mergeAnimations( const AnimationModeE & mode, const unsigned & anim1
    return false;
 }
 
-bool Model::moveAnimation( const AnimationModeE & mode, const unsigned & oldIndex, const unsigned & newIndex )
+bool Model::moveAnimation( AnimationModeE mode, unsigned oldIndex, unsigned newIndex )
 {
    if ( oldIndex == newIndex )
    {
@@ -873,7 +938,7 @@ bool Model::moveAnimation( const AnimationModeE & mode, const unsigned & oldInde
    return false;
 }
 
-int Model::convertAnimToFrame( const AnimationModeE & mode, const unsigned & anim, const char * newName, const unsigned & frameCount )
+int Model::convertAnimToFrame( AnimationModeE mode, unsigned anim, const char * newName, unsigned frameCount )
 {
    int num = -1;
    
@@ -886,6 +951,7 @@ int Model::convertAnimToFrame( const AnimationModeE & mode, const unsigned & ani
             if ( num >= 0 )
             {
                setAnimFrameCount( ANIMMODE_FRAME, num, frameCount );
+               setAnimLooping( ANIMMODE_FRAME, num, getAnimLooping( ANIMMODE_SKELETAL, anim ) );
 
                if ( frameCount > 0 )
                {
@@ -946,7 +1012,7 @@ int Model::convertAnimToFrame( const AnimationModeE & mode, const unsigned & ani
    return num;
 }
 
-bool Model::clearAnimFrame( const AnimationModeE & m, const unsigned & anim, const unsigned & frame )
+bool Model::clearAnimFrame( AnimationModeE m, unsigned anim, unsigned frame )
 {
    switch ( m )
    {
@@ -1004,8 +1070,8 @@ bool Model::clearAnimFrame( const AnimationModeE & m, const unsigned & anim, con
    return false;
 }
 
-int Model::setSkelAnimKeyframe( const unsigned & anim, const unsigned & frame, const unsigned & joint, const bool & isRotation,
-      const double & x, const double & y, const double & z )
+int Model::setSkelAnimKeyframe( unsigned anim, unsigned frame, unsigned joint, bool isRotation,
+      double x, double y, double z )
 {
    if ( anim < m_skelAnims.size() 
          && frame < m_skelAnims[anim]->m_frameCount
@@ -1084,7 +1150,7 @@ int Model::setSkelAnimKeyframe( const unsigned & anim, const unsigned & frame, c
    }
 }
 
-bool Model::deleteSkelAnimKeyframe( const unsigned & anim, const unsigned & frame, const unsigned & joint, const bool & isRotation )
+bool Model::deleteSkelAnimKeyframe( unsigned anim, unsigned frame, unsigned joint, bool isRotation )
 {
    if ( anim < m_skelAnims.size() && frame < m_skelAnims[anim]->m_frameCount )
    {
@@ -1115,7 +1181,7 @@ bool Model::deleteSkelAnimKeyframe( const unsigned & anim, const unsigned & fram
    return rval;
 }
 
-bool Model::insertSkelAnimKeyframe( const unsigned & anim, Keyframe * keyframe )
+bool Model::insertSkelAnimKeyframe( unsigned anim, Keyframe * keyframe )
 {
    if ( anim < m_skelAnims.size() 
          && keyframe->m_frame < m_skelAnims[anim]->m_frameCount 
@@ -1131,7 +1197,7 @@ bool Model::insertSkelAnimKeyframe( const unsigned & anim, Keyframe * keyframe )
    }
 }
 
-bool Model::removeSkelAnimKeyframe( const unsigned & anim, const unsigned & frame, const unsigned & joint, const bool & isRotation, bool release )
+bool Model::removeSkelAnimKeyframe( unsigned anim, unsigned frame, unsigned joint, bool isRotation, bool release )
 {
    if ( anim < m_skelAnims.size() && frame < m_skelAnims[anim]->m_frameCount )
    {
@@ -1160,7 +1226,7 @@ bool Model::removeSkelAnimKeyframe( const unsigned & anim, const unsigned & fram
    }
 }
 
-void Model::insertFrameAnim( const unsigned & index, FrameAnim * anim )
+void Model::insertFrameAnim( unsigned index, FrameAnim * anim )
 {
    LOG_PROFILE();
 
@@ -1188,7 +1254,7 @@ void Model::insertFrameAnim( const unsigned & index, FrameAnim * anim )
    }
 }
 
-void Model::removeFrameAnim( const unsigned & index )
+void Model::removeFrameAnim( unsigned index )
 {
    LOG_PROFILE();
 
@@ -1214,7 +1280,7 @@ void Model::removeFrameAnim( const unsigned & index )
    }
 }
 
-void Model::insertSkelAnim( const unsigned & index, SkelAnim * anim )
+void Model::insertSkelAnim( unsigned index, SkelAnim * anim )
 {
    LOG_PROFILE();
 
@@ -1242,7 +1308,7 @@ void Model::insertSkelAnim( const unsigned & index, SkelAnim * anim )
    }
 }
 
-void Model::removeSkelAnim( const unsigned & index )
+void Model::removeSkelAnim( unsigned index )
 {
    LOG_PROFILE();
 
@@ -1268,7 +1334,7 @@ void Model::removeSkelAnim( const unsigned & index )
    }
 }
 
-void Model::insertFrameAnimFrame( const unsigned & anim, const unsigned & frame, 
+void Model::insertFrameAnimFrame( unsigned anim, unsigned frame, 
       Model::FrameAnimData * data )
 {
    if ( anim < m_frameAnims.size() )
@@ -1278,7 +1344,7 @@ void Model::insertFrameAnimFrame( const unsigned & anim, const unsigned & frame,
       {
          fa->m_frameData.push_back( data );
       }
-      else if ( frame == fa->m_frameData.size() )
+      else if ( frame < fa->m_frameData.size() )
       {
          unsigned count = 0;
          vector<FrameAnimData *>::iterator it;
@@ -1299,7 +1365,7 @@ void Model::insertFrameAnimFrame( const unsigned & anim, const unsigned & frame,
    }
 }
 
-void Model::removeFrameAnimFrame( const unsigned & anim, const unsigned & frame )
+void Model::removeFrameAnimFrame( unsigned anim, unsigned frame )
 {
    if ( anim < m_frameAnims.size() && m_frameAnims[anim]->m_frameData.size() )
    {
@@ -1320,7 +1386,7 @@ void Model::removeFrameAnimFrame( const unsigned & anim, const unsigned & frame
 
 #endif // MM3D_EDIT
 
-bool Model::setCurrentAnimation( const AnimationModeE & m, const char * name )
+bool Model::setCurrentAnimation( AnimationModeE m, const char * name )
 {
 #ifdef MM3D_EDIT
    bool needUndo = m_animationMode ? false : true;
@@ -1337,6 +1403,8 @@ bool Model::setCurrentAnimation( const AnimationModeE & m, const char * name )
       m_changeBits |= AnimationMode;
    }
 
+   m_validBspTree = false;
+
    unsigned t;
    switch ( m )
    {
@@ -1470,7 +1538,7 @@ bool Model::setCurrentAnimation( const AnimationModeE & m, const char * name )
    return false;
 }
 
-bool Model::setCurrentAnimation( const AnimationModeE & m, unsigned index )
+bool Model::setCurrentAnimation( AnimationModeE m, unsigned index )
 {
 #ifdef MM3D_EDIT
    bool needUndo = (m_animationMode) ? false : true;
@@ -1485,6 +1553,8 @@ bool Model::setCurrentAnimation( const AnimationModeE & m, unsigned index )
       m_changeBits |= AnimationMode;
    }
 
+   m_validBspTree = false;
+
    log_debug( "Changing animation from %d to %d\n", oldMode, m );
 
    switch ( m )
@@ -1617,6 +1687,8 @@ bool Model::setCurrentAnimationFrame( unsigned frame )
 {
    m_changeBits |= AnimationFrame;
 
+   m_validBspTree = false;
+
    if ( m_animationMode == ANIMMODE_FRAME 
          && m_currentAnim < m_frameAnims.size() 
          && frame < m_frameAnims[m_currentAnim]->m_frameData.size() )
@@ -1651,6 +1723,8 @@ bool Model::setCurrentAnimationTime( double frameTime )
 
    m_changeBits |= AnimationFrame;
 
+   m_validBspTree = false;
+
    if ( m_animationMode == ANIMMODE_FRAME && m_currentAnim < m_frameAnims.size() )
    {
       if ( !m_frameAnims[m_currentAnim]->m_validNormals )
@@ -1666,7 +1740,7 @@ bool Model::setCurrentAnimationTime( double frameTime )
          double totalTime = spf * m_frameAnims[m_currentAnim]->m_frameData.size();
          while ( frameTime >= totalTime )
          {
-            if ( !m_animationLoop )
+            if ( !m_frameAnims[m_currentAnim]->m_loop )
             {
                return false;
             }
@@ -1695,7 +1769,7 @@ bool Model::setCurrentAnimationTime( double frameTime )
          double totalTime = sa->m_spf * sa->m_frameCount;
          while ( frameTime > totalTime )
          {
-            if ( !m_animationLoop )
+            if ( !sa->m_loop )
             {
                return false;
             }
@@ -1713,7 +1787,7 @@ bool Model::setCurrentAnimationTime( double frameTime )
       {
          Matrix transform;
          interpSkelAnimKeyframeTime( m_currentAnim, frameTime,
-               m_animationLoop, j, transform );
+               sa->m_loop, j, transform );
 
          Matrix relativeFinal( m_joints[j]->m_relative );
          relativeFinal = transform * relativeFinal;
@@ -1735,10 +1809,9 @@ bool Model::setCurrentAnimationTime( double frameTime )
          Vertex * vptr = m_vertices[v];
          if ( !vptr->m_influences.empty() )
          {
-            vptr->m_kfCoord[0] = 0;
-            vptr->m_kfCoord[1] = 0;
-            vptr->m_kfCoord[2] = 0;
-            //double fnorm[3] = { 0, 0, 0 };
+            vptr->m_kfCoord[0] = 0.0;
+            vptr->m_kfCoord[1] = 0.0;
+            vptr->m_kfCoord[2] = 0.0;
 
             double total = 0.0;
 
@@ -1747,31 +1820,40 @@ bool Model::setCurrentAnimationTime( double frameTime )
             InfluenceList::iterator it;
             for ( it = vptr->m_influences.begin(); it != vptr->m_influences.end(); it++ )
             {
-               const Matrix & final = m_joints[ (*it).m_boneId ]->m_final;
-               const Matrix & abs = m_joints[ (*it).m_boneId ]->m_absolute;
+               if ( it->m_weight > 0.00001 )
+               {
+                  const Matrix & final = m_joints[ (*it).m_boneId ]->m_final;
+                  const Matrix & abs = m_joints[ (*it).m_boneId ]->m_absolute;
 
-               vert.setAll( vptr->m_coord );
-               vert[3] = 1.0;
+                  vert.setAll( vptr->m_coord );
+                  vert[3] = 1.0;
 
-               abs.inverseTranslateVector( (double *) vert.getVector() );
-               abs.inverseRotateVector( (double *) vert.getVector() );
+                  abs.inverseTranslateVector( (double *) vert.getVector() );
+                  abs.inverseRotateVector( (double *) vert.getVector() );
 
-               vert.transform( final );
+                  vert.transform( final );
 
-               vert.scale3( (*it).m_weight );
-               vptr->m_kfCoord[0] += vert[0];
-               vptr->m_kfCoord[1] += vert[1];
-               vptr->m_kfCoord[2] += vert[2];
+                  vert.scale3( (*it).m_weight );
+                  vptr->m_kfCoord[0] += vert[0];
+                  vptr->m_kfCoord[1] += vert[1];
+                  vptr->m_kfCoord[2] += vert[2];
 
-               total += (*it).m_weight;
+                  total += (*it).m_weight;
+               }
             }
 
-            if ( total > 0.0 )
+            if ( total > 0.00001 )
             {
                vptr->m_kfCoord[0] /= total;
                vptr->m_kfCoord[1] /= total;
                vptr->m_kfCoord[2] /= total;
             }
+            else
+            {
+               vptr->m_kfCoord[0] = vptr->m_coord[0];
+               vptr->m_kfCoord[1] = vptr->m_coord[1];
+               vptr->m_kfCoord[2] = vptr->m_coord[2];
+            }
          }
          else
          {
@@ -1856,6 +1938,15 @@ bool Model::setCurrentAnimationTime( double frameTime )
                pptr->m_kfTrans[1] /= total;
                pptr->m_kfTrans[2] /= total;
             }
+            else
+            {
+               pptr->m_kfTrans[0] = 0.0;
+               pptr->m_kfTrans[1] = 0.0;
+               pptr->m_kfTrans[2] = 0.0;
+               pptr->m_kfRot[0] = 0.0;
+               pptr->m_kfRot[1] = 0.0;
+               pptr->m_kfRot[2] = 0.0;
+            }
 
             normalize3( axisX );
             normalize3( axisY );
@@ -1979,31 +2070,21 @@ bool Model::setCurrentAnimationTime( double frameTime )
    }
 }
 
-unsigned Model::getCurrentAnimation()
+unsigned Model::getCurrentAnimation() const
 {
    return m_currentAnim;
 }
 
-unsigned Model::getCurrentAnimationFrame()
+unsigned Model::getCurrentAnimationFrame() const
 {
    return m_currentFrame;
 }
 
-double Model::getCurrentAnimationTime()
+double Model::getCurrentAnimationTime() const
 {
    return m_currentTime;
 }
 
-void Model::setAnimationLooping( bool o )
-{
-   m_animationLoop = o;
-}
-
-bool Model::isAnimationLooping()
-{
-   return m_animationLoop;
-}
-
 void Model::setNoAnimation()
 {
 #ifdef MM3D_EDIT
@@ -2021,6 +2102,8 @@ void Model::setNoAnimation()
    if ( m_animationMode != ANIMMODE_NONE )
    {
       m_changeBits |= AnimationMode;
+
+      m_validBspTree = false;
    }
 
    unsigned vcount = m_vertices.size();
@@ -2029,12 +2112,12 @@ void Model::setNoAnimation()
       m_vertices[ v ]->m_drawSource = m_vertices[v]->m_coord;
    }
 
-      unsigned pcount = m_points.size();
-      for ( unsigned p = 0; p < pcount; p++ )
-      {
-         m_points[ p ]->m_drawSource = m_points[p]->m_trans;
-         m_points[ p ]->m_rotSource  = m_points[p]->m_rot;
-      }
+   unsigned pcount = m_points.size();
+   for ( unsigned p = 0; p < pcount; p++ )
+   {
+      m_points[ p ]->m_drawSource = m_points[p]->m_trans;
+      m_points[ p ]->m_rotSource  = m_points[p]->m_rot;
+   }
 
    unsigned tcount = m_triangles.size();
    for ( unsigned t = 0; t < tcount; t++ )
@@ -2087,40 +2170,18 @@ void Model::setupJoints()
 //            joint->m_localRotation[0], joint->m_localRotation[1], joint->m_localRotation[2] );
 //      log_debug( "\n" );
    }
-
-   /*
-   for ( unsigned p = 0; p < m_points.size(); p++ )
+   
+   for ( unsigned anim = 0; anim < m_skelAnims.size(); ++anim )
    {
-      Point * point = m_points[p];
-      double trans[4] = { point->m_trans[0], point->m_trans[1], point->m_trans[2], 1.0 };
-      double rot[3]   = { point->m_rot[0], point->m_rot[1], point->m_rot[2] };
-
-      if ( point->m_boneId >= 0 )
+      while ( m_joints.size() > m_skelAnims[anim]->m_jointKeyframes.size() )
       {
-         log_debug( "point absolute point position for %d is %f,%f,%f \n",
-                 p, trans[0], trans[1], trans[2] );
-         Matrix minv = m_joints[ point->m_boneId ]->m_final.getInverse();
-         minv.apply( trans );
-         log_debug( "point relative point position for %d is %f,%f,%f \n",
-                 p, trans[0], trans[1], trans[2] );
-         Matrix mr;
-         mr.setRotation( rot );
-         mr = mr * minv;
-         mr.getRotation( rot );
+         KeyframeList kl;
+         m_skelAnims[anim]->m_jointKeyframes.push_back( kl );
       }
-
-      for ( int t = 0; t < 3; t++ )
-      {
-         point->m_localTranslation[t] = trans[t];
-         point->m_localRotation[t]    = rot[t];
-      }
-      log_debug( "point final relative point position for %d is %f,%f,%f \n",
-              p, point->m_localTranslation[0], point->m_localTranslation[1], point->m_localTranslation[2] );
    }
-   */
 }
 
-unsigned Model::getAnimCount( const AnimationModeE & m ) const
+unsigned Model::getAnimCount( AnimationModeE m ) const
 {
    switch ( m )
    {
@@ -2136,7 +2197,7 @@ unsigned Model::getAnimCount( const AnimationModeE & m ) const
 
    return 0;
 }
-const char * Model::getAnimName( const AnimationModeE & m, const unsigned & anim )
+const char * Model::getAnimName( AnimationModeE m, unsigned anim ) const
 {
    switch ( m )
    {
@@ -2159,7 +2220,7 @@ const char * Model::getAnimName( const AnimationModeE & m, const unsigned & anim
    return NULL;
 }
 
-unsigned Model::getAnimFrameCount( const AnimationModeE & m, const unsigned & anim )
+unsigned Model::getAnimFrameCount( AnimationModeE m, unsigned anim ) const
 {
    switch ( m )
    {
@@ -2182,7 +2243,7 @@ unsigned Model::getAnimFrameCount( const AnimationModeE & m, const unsigned & an
    return 0;
 }
 
-double Model::getAnimFPS( const AnimationModeE & m, const unsigned & anim )
+double Model::getAnimFPS( AnimationModeE m, unsigned anim ) const
 {
    switch ( m )
    {
@@ -2205,8 +2266,31 @@ double Model::getAnimFPS( const AnimationModeE & m, const unsigned & anim )
    return 0;
 }
 
-bool Model::getFrameAnimVertexCoords( const unsigned & anim, const unsigned & frame, const unsigned & vertex,
-      double & x, double & y, double & z )
+bool Model::getAnimLooping( AnimationModeE m, unsigned anim ) const
+{
+   switch ( m )
+   {
+      case ANIMMODE_SKELETAL:
+         if ( anim < m_skelAnims.size() )
+         {
+            return m_skelAnims[anim]->m_loop;
+         }
+         break;
+      case ANIMMODE_FRAME:
+         if ( anim < m_frameAnims.size() )
+         {
+            return m_frameAnims[anim]->m_loop;
+         }
+         break;
+      default:
+         break;
+   }
+
+   return true;
+}
+
+bool Model::getFrameAnimVertexCoords( unsigned anim, unsigned frame, unsigned vertex,
+      double & x, double & y, double & z ) const
 {
    if ( anim < m_frameAnims.size() 
          && frame < m_frameAnims[anim]->m_frameData.size()
@@ -2231,8 +2315,8 @@ bool Model::getFrameAnimVertexCoords( const unsigned & anim, const unsigned & fr
    }
 }
 
-bool Model::getFrameAnimVertexNormal( const unsigned & anim, const unsigned & frame, const unsigned & vertex,
-      double & x, double & y, double & z )
+bool Model::getFrameAnimVertexNormal( unsigned anim, unsigned frame, unsigned vertex,
+      double & x, double & y, double & z ) const
 {
    if ( anim < m_frameAnims.size() 
          && frame < m_frameAnims[anim]->m_frameData.size()
@@ -2264,7 +2348,7 @@ bool Model::getFrameAnimVertexNormal( const unsigned & anim, const unsigned & fr
 }
 
 bool Model::hasSkelAnimKeyframe( unsigned anim, unsigned frame,
-      unsigned joint, bool isRotation )
+      unsigned joint, bool isRotation ) const
 {
    if ( anim < m_skelAnims.size() 
          && frame < m_skelAnims[anim]->m_frameCount
@@ -2300,7 +2384,7 @@ bool Model::hasSkelAnimKeyframe( unsigned anim, unsigned frame,
 
 bool Model::getSkelAnimKeyframe( unsigned anim, unsigned frame,
       unsigned joint, bool isRotation,
-      double & x, double & y, double & z )
+      double & x, double & y, double & z ) const
 {
    if ( anim < m_skelAnims.size() 
          && frame < m_skelAnims[anim]->m_frameCount
@@ -2346,7 +2430,7 @@ bool Model::getSkelAnimKeyframe( unsigned anim, unsigned frame,
 
 bool Model::interpSkelAnimKeyframe( unsigned anim, unsigned frame,
       bool loop, unsigned j, bool isRotation,
-      double & x, double & y, double & z )
+      double & x, double & y, double & z ) const
 {
    if ( anim < m_skelAnims.size() 
          && frame < m_skelAnims[anim]->m_frameCount
@@ -2376,7 +2460,7 @@ bool Model::interpSkelAnimKeyframe( unsigned anim, unsigned frame,
 }
 
 bool Model::interpSkelAnimKeyframeTime( unsigned anim, double frameTime,
-      bool loop, unsigned j, Matrix & transform )
+      bool loop, unsigned j, Matrix & transform ) const
 {
    if ( anim < m_skelAnims.size() && j < m_joints.size() )
    {
@@ -2487,6 +2571,13 @@ bool Model::interpSkelAnimKeyframeTime( unsigned anim, double frameTime,
 
                double tm = (tempTime - sa->m_jointKeyframes[j][rot]->m_time) / diff;
 
+               // Negate if necessary to get shortest rotation path for
+               // interpolation
+               if ( va.dot4(vb) < -0.00001 )
+               {
+                  vb[0] = -vb[0]; vb[1] = -vb[1]; vb[2] = -vb[2]; vb[3] = -vb[3];
+               }
+
                va = va * (1.0 - tm) + (vb * tm);
                va = va * (1.0 / va.mag());
             }
