@@ -1,4 +1,4 @@
-/*  Misfit Model 3D
+/*  Maverick Model 3D
  * 
  *  Copyright (c) 2004-2007 Kevin Worcester
  * 
@@ -727,6 +727,7 @@ void Model::removeTexture( unsigned index )
          if ( count == index )
          {
             m_materials.erase( it );
+            invalidateTextures();
             break;
          }
          count++;
