@@ -1,4 +1,4 @@
-/*  Misfit Model 3D
+/*  Maverick Model 3D
  * 
  *  Copyright (c) 2004-2007 Kevin Worcester
  * 
@@ -33,11 +33,14 @@
 #include "datadest.h"
 #include "datasource.h"
 
+#include "translate.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <errno.h>
 #include <string.h>
 #include <stdint.h>
+#include <limits.h>
 
 #include <string>
 #include <vector>
@@ -47,17 +50,6 @@ using std::string;
 
 char const Ms3dFilter::MAGIC_NUMBER[] = "MS3D000000";
 
-template<typename T>
-class FunctionCaller
-{
-   public:
-      FunctionCaller( T * obj, void (T::*method)(void) ) { m_obj = obj; m_method = method; }
-      ~FunctionCaller() { (m_obj->*m_method)(); }
-   private:
-      T * m_obj;
-      void (T::*m_method)(void);
-};
-
 /* 
 	MS3D STRUCTURES 
 */
@@ -149,6 +141,7 @@ typedef struct _JointNameListRec_t JointNameListRecT;
 Ms3dFilter::Ms3dOptions::Ms3dOptions()
    : m_subVersion( 0 ),
      m_vertexExtra( 0xffffffff ),
+     m_vertexExtra2( 0xffffffff ),
      m_jointColor( 0xffffffff )
 {
 }
@@ -163,7 +156,7 @@ void Ms3dFilter::Ms3dOptions::setOptionsFromModel( Model * m )
    if ( m->getMetaData( "ms3d_sub_version", value, sizeof(value) ) )
    {
       m_subVersion = atoi( value );
-      m_subVersion = util_clamp( m_subVersion, 0, 2 );
+      m_subVersion = util_clamp( m_subVersion, 0, 3 );
    }
    else
    {
@@ -177,7 +170,7 @@ void Ms3dFilter::Ms3dOptions::setOptionsFromModel( Model * m )
          m->getVertexInfluences( v, il );
          if ( il.size() > 1 )
          {
-            m_subVersion = 1;
+            m_subVersion = 2;
          }
       }
    }
@@ -187,6 +180,11 @@ void Ms3dFilter::Ms3dOptions::setOptionsFromModel( Model * m )
       sscanf( value, "%x", &m_vertexExtra );
    }
 
+   if ( m->getMetaData( "ms3d_vertex_extra2", value, sizeof(value) ) )
+   {
+      sscanf( value, "%x", &m_vertexExtra2 );
+   }
+
    // TODO joint color
    //if ( m->getMetaData( "ms3d_joint_color", value, sizeof(value) ) )
    //{
@@ -207,7 +205,7 @@ Model::ModelErrorE Ms3dFilter::readFile( Model * model, const char * const filen
 
    Model::ModelErrorE err = Model::ERROR_NONE;
    m_src = openInput( filename, err );
-   FunctionCaller<DataSource> fc( m_src, &DataSource::close );
+   SourceCloser fc( m_src );
 
    if ( err != Model::ERROR_NONE )
       return err;
@@ -271,7 +269,14 @@ Model::ModelErrorE Ms3dFilter::readFile( Model * model, const char * const filen
          vert->m_coord[v] = vertex.m_vertex[ v ];
       }
       modelVertices.push_back( vert );
-      vertexJoints.push_back( vertex.m_boneId );
+      if ( vertex.m_boneId == 0xFF )
+      {
+         vertexJoints.push_back( -1 );
+      }
+      else
+      {
+         vertexJoints.push_back( vertex.m_boneId );
+      }
    }
 
    uint16_t numTriangles = 0;
@@ -316,8 +321,8 @@ Model::ModelErrorE Ms3dFilter::readFile( Model * model, const char * const filen
       {
          if ( curTriangle->m_vertexIndices[i] >= numVertices )
          {
-            //log_debug( "vertex: %d/%d\n", curTriangle->m_vertexIndices[i],
-            //      modelVertices.size() );
+            log_error( "vertex out of range: %d/%d\n", curTriangle->m_vertexIndices[i],
+                  modelVertices.size() );
             return Model::ERROR_BAD_DATA;
          }
       }
@@ -369,16 +374,23 @@ Model::ModelErrorE Ms3dFilter::readFile( Model * model, const char * const filen
          m_src->read( triIndex );
          if ( triIndex >= modelTriangles.size() )
          {
-            log_error( "TRIANGLE OUT OF RANGE %d/%d\n",
+            log_error( "triangle out of range: %d/%d\n",
                   triIndex, modelTriangles.size() );
             return Model::ERROR_BAD_DATA;
          }
          group->m_triangleIndices.insert( triIndex );
       }
 
-      int8_t material = 0;
+      uint8_t material = 0;
       m_src->read( material );
-      group->m_materialIndex = material;
+      if ( material == 0xFF )
+      {
+         group->m_materialIndex = -1;
+      }
+      else
+      {
+         group->m_materialIndex = material;
+      }
 
       // Already added group to m_groups
    }
@@ -390,7 +402,11 @@ Model::ModelErrorE Ms3dFilter::readFile( Model * model, const char * const filen
    for ( t = 0; t < numGroups; t++ )
    {
       if ( modelGroups[t]->m_materialIndex >= numMaterials )
+      {
+         log_error( "material out of range: %d/%d\n",
+               modelGroups[t]->m_materialIndex, numMaterials );
          return Model::ERROR_BAD_DATA;
+      }
    }
 
    for ( t = 0; t < numMaterials; t++ )
@@ -473,8 +489,6 @@ Model::ModelErrorE Ms3dFilter::readFile( Model * model, const char * const filen
       modelMaterials.push_back( mat );
    }
 
-   setModelInitialized( model, true );
-
    float32_t fps = 0;
    m_src->read( fps );
 
@@ -485,8 +499,6 @@ Model::ModelErrorE Ms3dFilter::readFile( Model * model, const char * const filen
    int32_t numFrames = 0;
    m_src->read( numFrames );
 
-   setModelNumFrames( model, numFrames );
-
    if ( numFrames > 0 )
    {
       model->addAnimation( Model::ANIMMODE_SKELETAL, "Keyframe" );
@@ -596,7 +608,7 @@ Model::ModelErrorE Ms3dFilter::readFile( Model * model, const char * const filen
             mkeyframe->m_parameter[i] = keyframe.m_parameter[i];
          }
 
-         unsigned frame = (unsigned) (keyframe.m_time / (1.0 / fps)) - 1;
+         unsigned frame = (unsigned) lround(keyframe.m_time / (1.0 / fps)) - 1;
 
          model->setSkelAnimKeyframe( 0, frame, t, true,
                mkeyframe->m_parameter[0], mkeyframe->m_parameter[1], mkeyframe->m_parameter[2] );
@@ -621,7 +633,7 @@ Model::ModelErrorE Ms3dFilter::readFile( Model * model, const char * const filen
             mkeyframe->m_parameter[i] = keyframe.m_parameter[i];
          }
 
-         unsigned frame = (unsigned) (keyframe.m_time / (1.0 / fps)) - 1;
+         unsigned frame = (unsigned) lround(keyframe.m_time / (1.0 / fps)) - 1;
 
          model->setSkelAnimKeyframe( 0, frame, t, false,
                mkeyframe->m_parameter[0], mkeyframe->m_parameter[1], mkeyframe->m_parameter[2] );
@@ -669,6 +681,31 @@ Model::ModelErrorE Ms3dFilter::writeFile( Model * model, const char * const file
       return Model::ERROR_BAD_ARGUMENT;
    }
 
+   if ( model->getVertexCount() > USHRT_MAX ) {
+      model->setFilterSpecificError( transll( QT_TRANSLATE_NOOP( "LowLevel", "Too many vertexes for MS3D export (max 65,536)." ) ).c_str() );
+      return Model::ERROR_FILTER_SPECIFIC;
+   }
+
+   if ( model->getTriangleCount() > USHRT_MAX ) {
+      model->setFilterSpecificError( transll( QT_TRANSLATE_NOOP( "LowLevel", "Too many faces for MS3D export (max 65,536)." ) ).c_str() );
+      return Model::ERROR_FILTER_SPECIFIC;
+   }
+
+   if ( model->getGroupCount() > 255 ) {
+      model->setFilterSpecificError( transll( QT_TRANSLATE_NOOP( "LowLevel", "Too many groups for MS3D export (max 255)." ) ).c_str() );
+      return Model::ERROR_FILTER_SPECIFIC;
+   }
+
+   if ( model->getTextureCount() > 255 ) {
+      model->setFilterSpecificError( transll( QT_TRANSLATE_NOOP( "LowLevel", "Too many materials for MS3D export (max 255)." ) ).c_str() );
+      return Model::ERROR_FILTER_SPECIFIC;
+   }
+
+   if ( model->getBoneJointCount() > 255 ) {
+      model->setFilterSpecificError( transll( QT_TRANSLATE_NOOP( "LowLevel", "Too many bone joints for MS3D export (max 255)." ) ).c_str() );
+      return Model::ERROR_FILTER_SPECIFIC;
+   }
+
    // Check for identical bone joint names
    {
       unsigned c = model->getBoneJointCount();
@@ -678,13 +715,31 @@ Model::ModelErrorE Ms3dFilter::writeFile( Model * model, const char * const file
          {
             if ( strcmp( model->getBoneJointName( i ), model->getBoneJointName( j ) ) == 0 )
             {
-               model->setFilterSpecificError( "Bone joints must have unique names." );
+               model->setFilterSpecificError( transll( QT_TRANSLATE_NOOP( "LowLevel", "Bone joints must have unique names for MS3D export." ) ).c_str() );
                return Model::ERROR_FILTER_SPECIFIC;
             }
          }
       }
    }
 
+   // Groups don't share vertices with Milk Shape 3D. Create mesh list
+   // that has unique vertices for each group. UVs/normals are per-face
+   // vertex rather than per-vertex, so vertices do not have to have
+   // unique UVs or normals.
+
+   MeshList ml;
+   mesh_create_list( ml, model, Mesh::MO_Group | Mesh::MO_Material );
+
+   if ( mesh_list_vertex_count( ml ) > USHRT_MAX ) {
+      model->setFilterSpecificError( transll( QT_TRANSLATE_NOOP( "LowLevel", "Too many vertexes for MS3D export (max 65,536) after duplicating vertexes used by multiple groups or materials." ) ).c_str() );
+      return Model::ERROR_FILTER_SPECIFIC;
+   }
+
+   if ( mesh_list_face_count( ml ) > USHRT_MAX ) {
+      model->setFilterSpecificError( transll( QT_TRANSLATE_NOOP( "LowLevel", "Too many faces for MS3D export (max 65,536) after duplicating vertexes used by multiple groups or materials." ) ).c_str() );
+      return Model::ERROR_FILTER_SPECIFIC;
+   }
+
    Model::ModelErrorE rval = Model::ERROR_NONE;
 
    // Use dynamic cast to determine if the object is of the proper type
@@ -711,19 +766,11 @@ Model::ModelErrorE Ms3dFilter::writeFile( Model * model, const char * const file
 
    Model::ModelErrorE err = Model::ERROR_NONE;
    m_dst = openOutput( filename, err );
-   FunctionCaller<DataDest> fc( m_dst, &DataDest::close );
+   DestCloser fc( m_dst );
 
    if ( err != Model::ERROR_NONE )
       return err;
 
-   // Groups don't share vertices with Milk Shape 3D. Create mesh list
-   // that has unique vertices for each group. UVs/normals are per-face
-   // vertex rather than per-vertex, so vertices do not have to have
-   // unique UVs or normals.
-
-   MeshList ml;
-   mesh_create_list( ml, model, Mesh::MO_Group | Mesh::MO_Material );
-
    vector<Model::Vertex *>   & modelVertices  = getVertexList( model );
    vector<Model::Triangle *> & modelTriangles = getTriangleList( model );
    vector<Model::Group *>    & modelGroups    = getGroupList( model );
@@ -873,8 +920,8 @@ Model::ModelErrorE Ms3dFilter::writeFile( Model * model, const char * const file
          m_dst->write( index );
       }
 
-      uint8_t material = static_cast<uint8_t>(~0);
-      if ( grp )
+      uint8_t material = 0xFF;
+      if ( grp && grp->m_materialIndex >= 0 )
          material = grp->m_materialIndex;
       m_dst->write( material );
 
@@ -954,7 +1001,13 @@ Model::ModelErrorE Ms3dFilter::writeFile( Model * model, const char * const file
    }
 
    float32_t currentTime = 1.0;
-   int32_t numFrames = model->getNumFrames();
+   int32_t numFrames = 0;
+
+   unsigned animcount = model->getAnimCount( Model::ANIMMODE_SKELETAL );
+   for ( unsigned anim = 0; anim < animcount; anim++ )
+   {
+      numFrames += model->getAnimFrameCount( Model::ANIMMODE_SKELETAL, anim );
+   }
 
    double spf = 1.0 / fps;
 
@@ -994,6 +1047,7 @@ Model::ModelErrorE Ms3dFilter::writeFile( Model * model, const char * const file
       unsigned animcount = model->getAnimCount( Model::ANIMMODE_SKELETAL );
       unsigned framecount = 0;
       unsigned prevcount = 0;
+      bool loop;
 
       unsigned a = 0;
       unsigned f = 0;
@@ -1011,11 +1065,14 @@ Model::ModelErrorE Ms3dFilter::writeFile( Model * model, const char * const file
 
          for ( f = 0; f < framecount; f++ )
          {
-            if ( model->getSkelAnimKeyframe( a, f, t, true, x, y, z ) )
+            // force keyframes at beginning and end of animation
+            if ( model->getSkelAnimKeyframe( a, f, t, true, x, y, z )
+               || ( f == 0 || f == framecount-1 ) )
             {
                rotcount++;
             }
-            if ( model->getSkelAnimKeyframe( a, f, t, false, x, y, z ) )
+            if ( model->getSkelAnimKeyframe( a, f, t, false, x, y, z )
+               || ( f == 0 || f == framecount-1 ) )
             {
                transcount++;
             }
@@ -1045,10 +1102,13 @@ Model::ModelErrorE Ms3dFilter::writeFile( Model * model, const char * const file
       for ( a = 0; a < animcount; a++ )
       {
          framecount = model->getAnimFrameCount( Model::ANIMMODE_SKELETAL, a );
+         loop = model->getAnimLooping( Model::ANIMMODE_SKELETAL, a );
 
          for ( f = 0; f < framecount; f++ )
          {
-            if ( model->getSkelAnimKeyframe( a, f, t, true, x, y, z ) )
+            // force keyframes at beginning and end of animation
+            if ( model->getSkelAnimKeyframe( a, f, t, true, x, y, z )
+               || ( ( f == 0 || f == framecount-1 ) && model->interpSkelAnimKeyframe( a, f, loop, t, true, x, y, z ) ) )
             {
                MS3DKeyframe keyframe;
                keyframe.m_time = ((double) (prevcount + f + 1)) * spf;
@@ -1074,10 +1134,13 @@ Model::ModelErrorE Ms3dFilter::writeFile( Model * model, const char * const file
       for ( a = 0; a < animcount; a++ )
       {
          framecount = model->getAnimFrameCount( Model::ANIMMODE_SKELETAL, a );
+         loop = model->getAnimLooping( Model::ANIMMODE_SKELETAL, a );
 
          for ( f = 0; f < framecount; f++ )
          {
-            if ( model->getSkelAnimKeyframe( a, f, t, false, x, y, z ) )
+            // force keyframes at beginning and end of animation
+            if ( model->getSkelAnimKeyframe( a, f, t, false, x, y, z )
+               || ( ( f == 0 || f == framecount-1 ) && model->interpSkelAnimKeyframe( a, f, loop, t, false, x, y, z ) ) )
             {
                MS3DKeyframe keyframe;
                keyframe.m_time = ((double) (prevcount + f + 1)) * spf;
@@ -1094,25 +1157,33 @@ Model::ModelErrorE Ms3dFilter::writeFile( Model * model, const char * const file
                m_dst->write( keyframe.m_parameter[2] );
             }
          }
-      }
 
-      prevcount += framecount;
+         prevcount += framecount;
+      }
    }
 
    int32_t subVersion = m_options->m_subVersion;
-   if ( subVersion == 1 || subVersion == 2 )
+   if ( subVersion >= 1 )
    {
       // Remember some values in meta data
       char value[128];
       sprintf( value, "%d", subVersion );
       m_model->updateMetaData( "ms3d_sub_version", value );
 
-      if ( subVersion == 2 )
+      if ( subVersion >= 2 )
       {
          sprintf( value, "%x", m_options->m_vertexExtra );
          m_model->updateMetaData( "ms3d_vertex_extra", value );
       }
 
+      if ( subVersion >= 3 )
+      {
+         sprintf( value, "%x", m_options->m_vertexExtra2 );
+         m_model->updateMetaData( "ms3d_vertex_extra2", value );
+      }
+
+      model->operationComplete( transll( QT_TRANSLATE_NOOP( "LowLevel", "Set meta data for MS3D export" ) ).c_str() );
+
       writeCommentSection();
       writeVertexWeightSection( ml );
 
@@ -1145,7 +1216,7 @@ void Ms3dFilter::writeCommentSection()
 void Ms3dFilter::writeVertexWeightSection( const MeshList & ml )
 {
    int32_t subVersion = m_options->m_subVersion;
-   if ( subVersion < 1 || subVersion > 2 )
+   if ( subVersion < 1 || subVersion > 3 )
    {
       subVersion = 1;
    }
@@ -1159,14 +1230,9 @@ void Ms3dFilter::writeVertexWeightSection( const MeshList & ml )
    MeshList::const_iterator it;
    for ( it = ml.begin(); it != ml.end(); it++ )
    {
-      // FIXME remove debug
-      //fprintf( stderr, "writing influences for mesh\n" );
       int vcount = it->vertices.size();
       for ( int v = 0; v < vcount; v++ )
       {
-         // FIXME remove debug
-         //fprintf( stderr, " mesh vertex %d (%d) is model vertex %d\n", v, vwritten, it->vertices[v].v );
-
          ilist.clear();
          if (!m_model->getVertexInfluences( it->vertices[v].v, ilist ))
             log_error( "get influences failed for vertex %d\n",
@@ -1181,7 +1247,7 @@ void Ms3dFilter::writeVertexWeightSection( const MeshList & ml )
 
 void Ms3dFilter::writeJointColorSection()
 {
-   int32_t subVersion = m_options->m_subVersion;
+   int32_t subVersion = 1;
    m_dst->write( subVersion );
 
    log_debug( "writing joint color subversion %d\n", subVersion );
@@ -1201,7 +1267,7 @@ void Ms3dFilter::writeJointColorSection()
 void Ms3dFilter::writeVertexWeight( int subVersion,
       const Model::InfluenceList & ilist )
 {
-   int8_t boneId[4]  = { -1, -1, -1, -1 };
+   uint8_t boneId[4] = { 0xFF, 0xFF, 0xFF, 0xFF };
    uint8_t weight[4] = { 0, 0, 0, 0 };
    int rawWeight[4]  = { 0, 0, 0, 0 };
 
@@ -1212,9 +1278,9 @@ void Ms3dFilter::writeVertexWeight( int subVersion,
 
    for ( it = ilist.rbegin(); index < 4 && it != ilist.rend(); it++ )
    {
-      totalWeight += static_cast<int>(it->m_weight * 100.0);
       boneId[ index ] = it->m_boneId;
-      rawWeight[ index ] = static_cast<int>(it->m_weight * 100.0);
+      rawWeight[ index ] = (int) lround( it->m_weight * 100.0 );
+      totalWeight += rawWeight[ index ];
 
       index++;
    }
@@ -1225,13 +1291,37 @@ void Ms3dFilter::writeVertexWeight( int subVersion,
    for ( it = ilist.rbegin(); index < 4 && it != ilist.rend(); it++ )
    {
       if ( totalWeight > 0 )
-         weight[ index ] = (uint8_t) (rawWeight[ index ] * (double) maxWeight
-               / (double) totalWeight);
+         weight[ index ] = (uint8_t) lround( rawWeight[ index ] * (double) maxWeight
+               / (double) totalWeight );
       else
-         weight[ index ] = maxWeight / ilist.size();
+         weight[ index ] = 0;
+
       index++;
    }
 
+   // Fix weights if they adding up to maxWeight + 1 (and while I haven't seen it happen also handle up to 4 too many)
+   if ( weight[0] + weight[1] + weight[2] + weight[3] > maxWeight ) {
+      int index, extraWeight = ( weight[0] + weight[1] + weight[2] + weight[3] ) - maxWeight;
+
+      log_debug( "fixing vertex weights: %d, %d, %d, %d (total: %d != %d)\n", weight[0], weight[1], weight[2], weight[3], weight[0] + weight[1] + weight[2] + weight[3], maxWeight );
+      log_debug( "          raw weights: %d, %d, %d, %d\n", rawWeight[0], rawWeight[1], rawWeight[2], rawWeight[3] );
+
+      for ( index = 3; index >= 0; index-- )
+      {
+         if ( extraWeight > 0 && weight[index] > 0 )
+         {
+            weight[index] -= 1;
+            extraWeight -= 1;
+         }
+      }
+
+      log_debug( "          new weights: %d, %d, %d, %d (total: %d)\n", weight[0], weight[1], weight[2], weight[3], weight[0] + weight[1] + weight[2] + weight[3] );
+   }
+
+   if ( weight[0] + weight[1] + weight[2] + weight[3] != maxWeight ) {
+      log_warning( "write ms3d vertex weights: %d, %d, %d, %d (total: %d != %d)\n", weight[0], weight[1], weight[2], weight[3], weight[0] + weight[1] + weight[2] + weight[3], maxWeight );
+   }
+
    // Yes, this needs to start at 1 (one), boneId[0] is stored
    // earlier in the file
    m_dst->write( boneId[1] );
@@ -1239,16 +1329,21 @@ void Ms3dFilter::writeVertexWeight( int subVersion,
    m_dst->write( boneId[3] );
 
    // Yes, this needs to start at 0 (zero), weight[3] is implicit
-   log_debug( "write weights: %d, %d, %d\n", weight[0], weight[1], weight[2] );
+   //log_debug( "write weights: %d, %d, %d\n", weight[0], weight[1], weight[2] );
    m_dst->write( weight[0] );
    m_dst->write( weight[1] );
    m_dst->write( weight[2] );
 
-   if ( subVersion == 2 )
+   if ( subVersion >= 2 )
    {
       uint32_t extra = m_options->m_vertexExtra;
       m_dst->write( extra );
    }
+   if ( subVersion >= 3 )
+   {
+      uint32_t extra2 = m_options->m_vertexExtra2;
+      m_dst->write( extra2 );
+   }
 }
 
 void Ms3dFilter::readString( char * buf, size_t len )
@@ -1313,7 +1408,11 @@ bool Ms3dFilter::readVertexWeightSection()
    int32_t subVersion = 0;
    m_src->read( subVersion );
 
-   subVersion = util_clamp( subVersion, 1, 2 );
+   if ( subVersion < 1 || subVersion > 3 )
+   {
+      log_debug( "  sub version: %d (unknown)\n", subVersion );
+      return false;
+   }
 
    char value[128];
    sprintf( value, "%d", subVersion );
@@ -1349,11 +1448,11 @@ bool Ms3dFilter::readVertexWeight( int subVersion,
 {
    weightList.clear();
 
-   int8_t boneIds[4]  = { -1, -1, -1, -1 };
-   uint8_t weights[4] = {  0,  0,  0,  0 };
+   uint8_t boneIds[4] = { 0xFF, 0xFF, 0xFF, 0xFF };
+   uint8_t weights[4] = { 0, 0, 0, 0 };
 
-   size_t size = (subVersion == 2) ? 10 : 6;
-   if ( size > m_src->getRemaining() )
+   size_t size[4] = { 0, 6, 10, 14 };
+   if ( size[subVersion] > m_src->getRemaining() )
    {
       return false;
    }
@@ -1373,18 +1472,27 @@ bool Ms3dFilter::readVertexWeight( int subVersion,
       {
          weights[i] = (uint8_t) ((int) weights[i] * 100 / 255);
       }
-      //log_debug( "      read weights %d\n", boneIds[i+1] );
+      //log_debug( "      read weights %d\n", weights[i] );
    }
-   if ( subVersion == 2 )
+   if ( subVersion >= 2 )
    {
       int32_t extra = 0;
       m_src->read( extra ); // don't do anything with this
    }
+   if ( subVersion >= 3 )
+   {
+      int32_t extra2 = 0;
+      m_src->read( extra2 ); // don't do anything with this
+   }
+
+   //log_debug( "      vtx %d: b %d, %d, %d, %d; w %d, %d, %d, %d\n", vertex,
+   //   (int)boneIds[0], (int)boneIds[1], (int)boneIds[2], (int)boneIds[3],
+   //   (int)weights[0], (int)weights[1], (int)weights[2], (int)weights[3] );
 
    VertexWeightT vw;
 
    int total = 0;
-   for ( i = 0; boneIds[i] >= 0 && i < 4; i++ )
+   for ( i = 0; i < 4 && boneIds[i] != 0xFF; i++ )
    {
       vw.boneId = boneIds[i];
       if ( i < 3 )
