@@ -1,4 +1,4 @@
-/*  Misfit Model 3D
+/*  Maverick Model 3D
  * 
  *  Copyright (c) 2004-2007 Kevin Worcester
  * 
@@ -44,7 +44,7 @@ class QMainWindow;
 //
 // Note that commands (non-interactive model operations) are much easier to
 // implement.  There are also many similarities in how they are used by 
-// Misfit Model.  You should see the documentation in command.h before 
+// Maverick Model 3D.  You should see the documentation in command.h before 
 // reading this.
 //
 // Note that middle mouse button events are used by ModelViewport and
@@ -231,7 +231,7 @@ class Tool
               double x, double y, double z );
 
       void makeToolCoordList( Parent * parent, ToolCoordList & list, 
-              const list< Model::Position > & positions );
+              const std::list< Model::Position > & positions );
 };
 
 class ToolSeparator : public Tool
