@@ -1,4 +1,4 @@
-/*  Misfit Model 3D
+/*  Maverick Model 3D
  * 
  *  Copyright (c) 2004-2007 Kevin Worcester
  * 
@@ -26,10 +26,25 @@
 
 #include "config.h"
 
-#define VERSION_MAJOR 1  // If you change version, change ../configure.in also
-#define VERSION_MINOR 3
-#define VERSION_PATCH 7
+// If you change the version, also change the following files
+// (relative to top source directory)
+//   configure.ac (for x.x.x)
+//   src/win_resource.rc (for x.x.x)
+//   mm3d-win32-installer.nsi (for x.x.x)
+//   Makefile.am (for Contents/PlugIns/mm3d/x.x)
+//   Makefile.generic (for plugins/x.x)
+//   plugins/Makefile.am (for plugins/x.x)
 
+#ifndef VERSION
+#define VERSION "1.3.11"
+#endif
+
+#define VERSION_MAJOR 1
+#define VERSION_MINOR 3
+#define VERSION_PATCH 11
+
+// Set to 0 for stable releases (even minor version) and 1 for
+// development versions (odd minor version)
 #define MM3D_DEVEL_VERSION 1
 
 #define VERSION_STRING VERSION
